@@ -29,16 +29,10 @@
 
 #define QQ_CHARSET_DEFAULT      "GB18030"
 
-gint convert_as_pascal_string(guint8 *data, gchar **ret, const gchar *from_charset);
-
-gchar *qq_smiley_to_purple(gchar *text);
-
-gchar *purple_smiley_to_qq(gchar *text);
+gint qq_get_vstr(gchar **ret, const gchar *from_charset, guint8 *data);
+gint qq_put_vstr(guint8 *buf, const gchar *str_utf8, const gchar *to_charset);
 
 gchar *utf8_to_qq(const gchar *str, const gchar *to_charset);
 gchar *qq_to_utf8(const gchar *str, const gchar *from_charset);
-gchar *qq_encode_to_purple(guint8 *font_attr_data, gint len, const gchar *msg);
 
-gchar *qq_im_filter_html(const gchar *text);
-void qq_filter_str(gchar *str);
 #endif
